@@ -20,8 +20,21 @@
 #include <stdio.h>
 #include <ctype.h>
 #include "getopt.h"
-#include "tilde/tilde.h"
 #include "bash.h"
+#include <ctype.h>
+#include <limits.h>
+
+// Defined in tilde.c.
+char *tilde_expand(const char *string);
+
+#ifdef _WIN32
+#include <errno.h>
+#define DEFAULT_PATHEXT ".exe;.ps1;.bat;.cmd;.com;"
+#define LEN_DEFAULT_PATHEXT sizeof(DEFAULT_PATHEXT)
+#define PATHEXTSEPARATORS ";"
+#include <windows.h>
+#define PATH_MAX MAX_PATH
+#endif /* WIN32 */
 
 static const char *progname;
 
@@ -52,7 +65,7 @@ static void print_usage(FILE *out)
 
 static void print_version(void)
 {
-  fprintf(stdout, "GNU which v" VERSION ", Copyright (C) 1999 - 2008 Carlo Wood.\n");
+  fprintf(stdout, "GNU which v" VERSION ", Copyright (C) 1999 - 2025 Carlo Wood.\n");
   fprintf(stdout, "GNU which comes with ABSOLUTELY NO WARRANTY;\n");
   fprintf(stdout, "This program is free software; your freedom to use, change\n");
   fprintf(stdout, "and distribute this program is protected by the GPL.\n");
@@ -63,10 +76,10 @@ static void print_fail(const char *name, const char *path_list)
   fprintf(stderr, "%s: no %s in (%s)\n", progname, name, path_list);
 }
 
-static char home[256];
+static char home[PATH_MAX];
 static size_t homelen = 0;
 
-static int absolute_path_given;
+static int path_containing_separator_given;
 static int found_path_starts_with_dot;
 static char *abs_path;
 
@@ -81,30 +94,31 @@ static char *find_command_in_path(const char *name, const char *path_list, int *
 
   name_len = strlen(name);
 
-  if (!absolute_program(name))
-    absolute_path_given = 0;
+  if (!contains_separator(name))
+    path_containing_separator_given = 0;
   else
   {
     char *p;
-    absolute_path_given = 1;
+    path_containing_separator_given = 1;
 
     if (abs_path)
       free(abs_path);
 
-    if (*name != '.' && *name != '/' && *name != '~')
+    if (*name != '.' && !IS_ABSOLUTE(name) && *name != '~')
     {
-      abs_path = (char *)xmalloc(3 + name_len);
+      abs_path = (char *) xmalloc(3 + name_len);
       strcpy(abs_path, "./");
+      abs_path[1] = DIR_SEPARATOR;
       strcat(abs_path, name);
     }
     else
     {
-      abs_path = (char *)xmalloc(1 + name_len);
+      abs_path = (char *) xmalloc(1 + name_len);
       strcpy(abs_path, name);
     }
 
     path_list = abs_path;
-    p = strrchr(abs_path, '/');
+    p = strrchr(abs_path, DIR_SEPARATOR);
     *p++ = 0;
     name = p;
   }
@@ -112,8 +126,7 @@ static char *find_command_in_path(const char *name, const char *path_list, int *
   while (path_list && path_list[*path_index])
   {
     char *path;
-
-    if (absolute_path_given)
+    if (path_containing_separator_given)
     {
       path = savestring(path_list);
       *path_index = strlen(path);
@@ -132,37 +145,73 @@ static char *find_command_in_path(const char *name, const char *path_list, int *
 
       if (skip_tilde)
       {
-	free(path);
-	continue;
+        free(path);
+        continue;
       }
     }
 
-    if (skip_dot && *path != '/')
+    if (skip_dot && !IS_ABSOLUTE(path))
     {
       free(path);
       continue;
     }
 
     found_path_starts_with_dot = (*path == '.');
-
     full_path = make_full_pathname(path, name, name_len);
-    free(path);
 
     status = file_status(full_path);
-
     if ((status & FS_EXISTS) && (status & FS_EXECABLE))
     {
       found = full_path;
+      free(path);
       break;
     }
 
+/* On MS-Windows also search for program name with executable extensions added */
+#ifdef _WIN32
+    if (!found && !strrchr(name, '.'))
+    {
+      LPTSTR PathExtStr, ext;
+      char *namex, *lasts;
+      int ext_len;
+
+      PathExtStr = strdup(getenv("PATHEXT"));
+      if (!PathExtStr)
+        PathExtStr = strdup(DEFAULT_PATHEXT);
+      ext = strtok_r(PathExtStr, PATHEXTSEPARATORS, &lasts);
+      while (ext)
+      {
+        free(full_path);
+        ext_len = strlen(ext);
+        namex = (char *) xmalloc(name_len + ext_len + 1);
+        if (!namex)
+          continue;
+        strcpy(namex, name);
+        strcat(namex, ext);
+        full_path = make_full_pathname(path, namex, name_len + ext_len);
+        free(namex);
+        status = file_status(full_path);
+        if ((status & FS_EXISTS) && (status & FS_EXECABLE))
+        {
+          found = full_path;
+          break;
+        }
+        ext = strtok_r(NULL, PATHEXTSEPARATORS, &lasts);
+      }
+      if (PathExtStr)
+        free(PathExtStr);
+    }
+#endif
+    free(path);
+    if (found)
+      break;
     free(full_path);
   }
 
   return (found);
 }
 
-static char cwd[256];
+static char cwd[PATH_MAX + 1];
 static size_t cwdlen;
 
 static void get_current_working_directory(void)
@@ -177,7 +226,7 @@ static void get_current_working_directory(void)
       strcpy(cwd, pwd);
   }
 
-  if (*cwd != '/')
+  if (!IS_ABSOLUTE(cwd))
   {
     fprintf(stderr, "Can't get current working directory\n");
     exit(-1);
@@ -185,23 +234,23 @@ static void get_current_working_directory(void)
 
   cwdlen = strlen(cwd);
 
-  if (cwd[cwdlen - 1] != '/')
+  if (!IS_DIRSEP(cwd[cwdlen - 1]))
   {
-    cwd[cwdlen++] = '/';
+    cwd[cwdlen++] = DIR_SEPARATOR;
     cwd[cwdlen] = 0;
   }
 }
 
 static char *path_clean_up(const char *path)
 {
-  static char result[256];
+  static char result[PATH_MAX];
 
   const char *p1 = path;
   char *p2 = result;
 
   int saw_slash = 0, saw_slash_dot = 0, saw_slash_dot_dot = 0;
 
-  if (*p1 != '/')
+  if (!IS_ABSOLUTE(p1))
   {
     get_current_working_directory();
     strcpy(result, cwd);
@@ -215,29 +264,29 @@ static char *path_clean_up(const char *path)
      * Two leading slashes are allowed, having an OS implementation-defined meaning.
      * See http://www.opengroup.org/onlinepubs/009695399/basedefs/xbd_chap04.html#tag_04_11
      */
-    if (!saw_slash || *p1 != '/' || (p1 == path + 1 && p1[1] != '/'))
+    if (!saw_slash || !IS_DIRSEP(*p1) || (p1 == path + 1 && !IS_DIRSEP(p1[1])))
       *p2++ = *p1;
-    if (saw_slash_dot && (*p1 == '/'))
+    if (saw_slash_dot && (IS_DIRSEP(*p1)))
       p2 -= 2;
-    if (saw_slash_dot_dot && (*p1 == '/'))
+    if (saw_slash_dot_dot && (IS_DIRSEP(*p1)))
     {
       int cnt = 0;
       do
       {
-	if (--p2 < result)
-	{
-	  strcpy(result, path);
-	  return result;
-	}
-	if (*p2 == '/')
-	  ++cnt;
+        if (--p2 < result)
+        {
+          strcpy(result, path);
+          return result;
+        }
+        if (!IS_DIRSEP(*p2))
+          ++cnt;
       }
       while (cnt != 3);
       ++p2;
     }
     saw_slash_dot_dot = saw_slash_dot && (*p1 == '.');
     saw_slash_dot = saw_slash && (*p1 == '.');
-    saw_slash = (*p1 == '/');
+    saw_slash = (IS_DIRSEP(*p1));
   }
   while (*p1++);
 
@@ -264,19 +313,19 @@ int func_search(int indent, const char *cmd, struct function_st *func_list, int
   int i;
   for (i = 0; i < func_count; ++i)
   {
-    if (!strcmp(functions[i].name, cmd))
+    if (!STRCMP(functions[i].name, cmd))
     {
       int j;
       if (indent)
         fputc('\t', stdout);
       if (function_start_type == 1)
-	fprintf(stdout, "%s () {\n", cmd);
+        fprintf(stdout, "%s () {\n", cmd);
       else
-	fprintf(stdout, "%s ()\n", cmd);
+        fprintf(stdout, "%s ()\n", cmd);
       for (j = 0; j < functions[i].line_count; ++j)
       {
-	if (indent)
-	  fputc('\t', stdout);
+        if (indent)
+          fputc('\t', stdout);
         fputs(functions[i].lines[j], stdout);
       }
       return 1;
@@ -300,35 +349,35 @@ int path_search(int indent, const char *cmd, const char *path_list)
       result = find_command_in_path(cmd, path_list, &path_index);
       if (result)
       {
-	const char *full_path = path_clean_up(result);
-	int in_home = (show_tilde || skip_tilde) && !strncmp(full_path, home, homelen);
-	if (indent)
-	  fprintf(stdout, "\t");
-	if (!(skip_tilde && in_home) && show_dot && found_path_starts_with_dot && !strncmp(full_path, cwd, cwdlen))
-	{
-	  full_path += cwdlen;
-	  fprintf(stdout, "./");
-	}
-	else if (in_home)
-	{
-	  if (skip_tilde)
-	  {
-	    next = 1;
-	    free(result);
-	    continue;
-	  }
-	  if (show_tilde)
-	  {
-	    full_path += homelen;
-	    fprintf(stdout, "~/");
-	  }
-	}
-	fprintf(stdout, "%s\n", full_path);
-	free(result);
-	found_something = 1;
+        const char *full_path = path_clean_up(result);
+        int in_home = (show_tilde || skip_tilde) && !STRNCMP(full_path, home, homelen);
+        if (indent)
+          fprintf(stdout, "\t");
+        if (!(skip_tilde && in_home) && show_dot && found_path_starts_with_dot && !STRNCMP(full_path, cwd, cwdlen))
+        {
+          full_path += cwdlen;
+          fprintf(stdout, ".%c", DIR_SEPARATOR);
+        }
+        else if (in_home)
+        {
+          if (skip_tilde)
+          {
+            next = 1;
+            free(result);
+            continue;
+          }
+          if (show_tilde)
+          {
+            full_path += homelen;
+            fprintf(stdout, "~%c", DIR_SEPARATOR);
+          }
+        }
+        fprintf(stdout, "%s\n", full_path);
+        free(result);
+        found_something = 1;
       }
       else
-	break;
+        break;
     }
     while (next);
   }
@@ -341,13 +390,13 @@ void process_alias(const char *str, int argc, char *argv[], const char *path_lis
   const char *p = str;
   int len = 0;
 
-  while(*p == ' ' || *p == '\t')
+  while (*p == ' ' || *p == '\t')
     ++p;
-  if (!strncmp("alias", p, 5))
+  if (!STRNCMP("alias", p, 5))
     p += 5;
-  while(*p == ' ' || *p == '\t')
+  while (*p == ' ' || *p == '\t')
     ++p;
-  while(*p && *p != ' ' && *p != '\t' && *p != '=')
+  while (*p && *p != ' ' && *p != '\t' && *p != '=')
     ++p, ++len;
 
   for (; argc > 0; --argc, ++argv)
@@ -355,7 +404,7 @@ void process_alias(const char *str, int argc, char *argv[], const char *path_lis
     char q = 0;
     char *cmd;
 
-    if (!*argv || len != strlen(*argv) || strncmp(*argv, &p[-len], len))
+    if (!*argv || len != strlen(*argv) || STRNCMP(*argv, &p[-len], len))
       continue;
 
     fputs(str, stdout);
@@ -363,37 +412,37 @@ void process_alias(const char *str, int argc, char *argv[], const char *path_lis
     if (!show_all)
       *argv = NULL;
 
-    while(*p == ' ' || *p == '\t')
+    while (*p == ' ' || *p == '\t')
       ++p;
     if (*p == '=')
       ++p;
-    while(*p == ' ' || *p == '\t')
+    while (*p == ' ' || *p == '\t')
       ++p;
     if (*p == '"' || *p == '\'')
       q = *p, ++p;
 
-    for(;;)
+    for (;;)
     {
       int found = 0;
 
-      while(*p == ' ' || *p == '\t')
-	++p;
+      while (*p == ' ' || *p == '\t')
+        ++p;
       len = 0;
-      while(*p && *p != ' ' && *p != '\t' && *p != q && *p != '|' && *p != '&')
-	++p, ++len;
+      while (*p && *p != ' ' && *p != '\t' && *p != q && *p != '|' && *p != '&')
+        ++p, ++len;
 
-      cmd = (char *)xmalloc(len + 1);
+      cmd = (char *) xmalloc(len + 1);
       strncpy(cmd, &p[-len], len);
       cmd[len] = 0;
-      if (*argv && !strcmp(cmd, *argv))
+      if (*argv && !STRCMP(cmd, *argv))
         *argv = NULL;
-      if (read_functions && !strchr(cmd, '/'))
+      if (read_functions && !strchr(cmd, DIR_SEPARATOR))
         found = func_search(1, cmd, functions, function_start_type);
       if (show_all || !found)
-	path_search(1, cmd, path_list);
+        path_search(1, cmd, path_list);
       free(cmd);
 
-      while(*p && (*p != '|' || p[1] == '|') && (*p != '&' || p[1] == '&'))
+      while (*p && (*p != '|' || p[1] == '|') && (*p != '&' || p[1] == '&'))
         ++p;
 
       if (!*p)
@@ -428,75 +477,83 @@ static uid_t const superuser = 0;
 
 int main(int argc, char *argv[])
 {
-  const char *path_list = getenv("PATH");
+  char *path_list = getenv("PATH");
   int short_option, fail_count = 0;
   static int long_option;
-  struct option longopts[] = {
-    {"help", 0, &long_option, opt_help},
-    {"version", 0, &long_option, opt_version},
-    {"skip-dot", 0, &long_option, opt_skip_dot},
-    {"skip-tilde", 0, &long_option, opt_skip_tilde},
-    {"show-dot", 0, &long_option, opt_show_dot},
-    {"show-tilde", 0, &long_option, opt_show_tilde},
-    {"tty-only", 0, &long_option, opt_tty_only},
-    {"all", 0, NULL, 'a'},
-    {"read-alias", 0, NULL, 'i'},
-    {"skip-alias", 0, &long_option, opt_skip_alias},
-    {"read-functions", 0, &long_option, opt_read_functions},
-    {"skip-functions", 0, &long_option, opt_skip_functions},
-    {NULL, 0, NULL, 0}
-  };
+  struct option longopts[] = {{"help", 0, &long_option, opt_help},
+                              {"version", 0, &long_option, opt_version},
+                              {"skip-dot", 0, &long_option, opt_skip_dot},
+                              {"skip-tilde", 0, &long_option, opt_skip_tilde},
+                              {"show-dot", 0, &long_option, opt_show_dot},
+                              {"show-tilde", 0, &long_option, opt_show_tilde},
+                              {"tty-only", 0, &long_option, opt_tty_only},
+                              {"all", 0, NULL, 'a'},
+                              {"read-alias", 0, NULL, 'i'},
+                              {"skip-alias", 0, &long_option, opt_skip_alias},
+                              {"read-functions", 0, &long_option, opt_read_functions},
+                              {"skip-functions", 0, &long_option, opt_skip_functions},
+                              {NULL, 0, NULL, 0}};
 
   progname = argv[0];
+#ifdef _WIN32
+  path_list = (char *) malloc(2 + strlen(path_list) + 1);
+  path_list[0] = '.';
+  path_list[1] = PATH_SEPARATOR;
+  path_list[2] = '\0';
+  strcat(path_list, getenv("PATH"));
+  /* omit final ';' */
+  if (path_list[strlen(path_list) - 1] == PATH_SEPARATOR)
+    path_list[strlen(path_list) - 1] = '\0';
+#endif /* _WIN32 */
   while ((short_option = getopt_long(argc, argv, "aivV", longopts, NULL)) != -1)
   {
     switch (short_option)
     {
-      case 0:
-	switch (long_option)
-	{
-	  case opt_help:
-	    print_usage(stdout);
-	    return 0;
-	  case opt_version:
-	    print_version();
-	    return 0;
-	  case opt_skip_dot:
-	    skip_dot = !tty_only;
-	    break;
-	  case opt_skip_tilde:
-	    skip_tilde = !tty_only;
-	    break;
-	  case opt_skip_alias:
-	    skip_alias = 1;
-	    break;
-	  case opt_show_dot:
-	    show_dot = !tty_only;
-	    break;
-	  case opt_show_tilde:
-	    show_tilde = (!tty_only && geteuid() != superuser);
-	    break;
-	  case opt_tty_only:
-	    tty_only = !isatty(1);
-	    break;
-	  case opt_read_functions:
-	    read_functions = 1;
-	    break;
-	  case opt_skip_functions:
-	    skip_functions = 1;
-	    break;
-	}
-	break;
-      case 'a':
-	show_all = 1;
-	break;
-      case 'i':
-        read_alias = 1;
-	break;
-      case 'v':
-      case 'V':
-	print_version();
-	return 0;
+    case 0:
+      switch (long_option)
+      {
+      case opt_help:
+        print_usage(stdout);
+        return 0;
+      case opt_version:
+        print_version();
+        return 0;
+      case opt_skip_dot:
+        skip_dot = !tty_only;
+        break;
+      case opt_skip_tilde:
+        skip_tilde = !tty_only;
+        break;
+      case opt_skip_alias:
+        skip_alias = 1;
+        break;
+      case opt_show_dot:
+        show_dot = !tty_only;
+        break;
+      case opt_show_tilde:
+        show_tilde = (!tty_only && geteuid() != superuser);
+        break;
+      case opt_tty_only:
+        tty_only = !isatty(1);
+        break;
+      case opt_read_functions:
+        read_functions = 1;
+        break;
+      case opt_skip_functions:
+        skip_functions = 1;
+        break;
+      }
+      break;
+    case 'a':
+      show_all = 1;
+      break;
+    case 'i':
+      read_alias = 1;
+      break;
+    case 'v':
+    case 'V':
+      print_version();
+      return 0;
     }
   }
 
@@ -515,9 +572,10 @@ int main(int argc, char *argv[])
     strncpy(home, h, sizeof(home));
     home[sizeof(home) - 1] = 0;
     homelen = strlen(home);
-    if (home[homelen - 1] != '/' && homelen < sizeof(home) - 1)
+    if (!IS_DIRSEP(home[homelen - 1]) && homelen < sizeof(home) - 1)
     {
-      strcat(home, "/");
+      home[homelen] = DIR_SEPARATOR;
+      home[homelen + 1] = '\0';
       ++homelen;
     }
   }
@@ -546,7 +604,8 @@ int main(int argc, char *argv[])
     if (isatty(0))
     {
       fprintf(stderr, "%s: %s: Warning: stdin is a tty.\n", progname,
-          (read_functions ? read_alias ? "--read-functions, --read-alias, -i" : "--read-functions" : "--read-alias, -i"));
+              (read_functions ? read_alias ? "--read-functions, --read-alias, -i" : "--read-functions"
+                              : "--read-alias, -i"));
     }
 
     while (fgets(buf, sizeof(buf), stdin))
@@ -555,102 +614,102 @@ int main(int argc, char *argv[])
       int function_start_has_declare;
       if (read_functions)
       {
-	// bash version 2.0.5a and older output a pattern for `str' like
-	// declare -fx FUNCTION_NAME ()
-	// {
-	//   body
-	// }
-	//
-	// bash version 2.0.5b and later output a pattern for `str' like
-	// FUNCTION_NAME ()
-	// {
-	//   body
-	// }
-	char *p = buf + strlen(buf) - 1;
-	while (isspace(*p) && p > buf + 2)
-	  --p;
-	if (*p == ')' && p[-1] == '(' && p[-2] == ' ')
-	{
-	  looks_like_function_start = 1;
-	  function_start_has_declare = (strncmp("declare -", buf, 9) == 0);
-	}
-	// Add some zsh support here.
-	// zsh does output a pattern for `str' like
-	// FUNCTION () {
-	//   body
-	// }
-	if (p > buf + 4 && *p == '{' && p[-1] == ' ' &&
-	    p[-2] == ')' && p[-3] == '(' && p[-4] == ' ')
-	{
-	  looks_like_function_start = 1;
-	  function_start_type = 1;
-	  function_start_has_declare = 0;
-	}
+        // bash version 2.0.5a and older output a pattern for `str' like
+        // declare -fx FUNCTION_NAME ()
+        // {
+        //   body
+        // }
+        //
+        // bash version 2.0.5b and later output a pattern for `str' like
+        // FUNCTION_NAME ()
+        // {
+        //   body
+        // }
+        char *p = buf + strlen(buf) - 1;
+        while (isspace(*p) && p > buf + 2)
+          --p;
+        if (*p == ')' && p[-1] == '(' && p[-2] == ' ')
+        {
+          looks_like_function_start = 1;
+          function_start_has_declare = (STRNCMP("declare -", buf, 9) == 0);
+        }
+        // Add some zsh support here.
+        // zsh does output a pattern for `str' like
+        // FUNCTION () {
+        //   body
+        // }
+        if (p > buf + 4 && *p == '{' && p[-1] == ' ' && p[-2] == ')' && p[-3] == '(' && p[-4] == ' ')
+        {
+          looks_like_function_start = 1;
+          function_start_type = 1;
+          function_start_has_declare = 0;
+        }
       }
       if (processing_aliases && !looks_like_function_start)
       {
-	// bash version 2.0.5b can throw in lines like "declare -fx FUNCTION_NAME", eat them.
-	if (!strncmp("declare -", buf, 9))
-	  continue;
-	if (alias_count == max_alias_count)
-	{
-	  max_alias_count += 32;
-	  aliases = (char **)xrealloc(aliases, max_alias_count * sizeof(char *));
-	}
-	aliases[alias_count++] = strcpy((char *)xmalloc(strlen(buf) + 1), buf);
+        // bash version 2.0.5b can throw in lines like "declare -fx FUNCTION_NAME", eat them.
+        if (!STRNCMP("declare -", buf, 9))
+          continue;
+        if (alias_count == max_alias_count)
+        {
+          max_alias_count += 32;
+          aliases = (char **) xrealloc(aliases, max_alias_count * sizeof(char *));
+        }
+        aliases[alias_count++] = strcpy((char *) xmalloc(strlen(buf) + 1), buf);
       }
       else if (read_functions && looks_like_function_start)
       {
         struct function_st *function;
         int max_line_count;
 
-	const char *p = buf;
-	int len = 0;
+        const char *p = buf;
+        int len = 0;
 
         processing_aliases = 0;
 
-	// Eat "declare -fx " at start of bash version 2.0.5a and older, if present.
-	if (function_start_has_declare)
-	{
-	  p += 9;
-	  while(*p && *p++ != ' ');
-	}
+        // Eat "declare -fx " at start of bash version 2.0.5a and older, if present.
+        if (function_start_has_declare)
+        {
+          p += 9;
+          while (*p && *p++ != ' ')
+            ;
+        }
 
-	while(*p && *p != ' ')
-	  ++p, ++len;
+        while (*p && *p != ' ')
+          ++p, ++len;
 
-	if (func_count == max_func_count)
-	{
-	  max_func_count += 16;
-	  functions = (struct function_st *)xrealloc(functions, max_func_count * sizeof(struct function_st));
-	}
-	function = &functions[func_count++];
-	function->name = (char *)xmalloc(len + 1);
-	strncpy(function->name, &p[-len], len);
-	function->name[len] = 0;
-	function->len = len;
-	max_line_count = 32;
-	function->lines = (char **)xmalloc(max_line_count * sizeof(char *));
-	function->line_count = 0;
-	while (fgets(buf, sizeof(buf), stdin))
-	{
-	  size_t blen = strlen(buf);
-	  function->lines[function->line_count++] = strcpy((char *)xmalloc(blen + 1), buf);
-	  if (!strcmp(buf, "}\n"))
-	    break;
+        if (func_count == max_func_count)
+        {
+          max_func_count += 16;
+          functions = (struct function_st *) xrealloc(functions, max_func_count * sizeof(struct function_st));
+        }
+        function = &functions[func_count++];
+        function->name = (char *) xmalloc(len + 1);
+        strncpy(function->name, &p[-len], len);
+        function->name[len] = 0;
+        function->len = len;
+        max_line_count = 32;
+        function->lines = (char **) xmalloc(max_line_count * sizeof(char *));
+        function->line_count = 0;
+        while (fgets(buf, sizeof(buf), stdin))
+        {
+          size_t blen = strlen(buf);
+          function->lines[function->line_count++] = strcpy((char *) xmalloc(blen + 1), buf);
+          if (!strcmp(buf, "}\n"))
+            break;
           if (function->line_count == max_line_count)
-	  {
-	    max_line_count += 32;
-	    function->lines = (char **)xrealloc(function->lines, max_line_count * sizeof(char *));
-	  }
-	}
+          {
+            max_line_count += 32;
+            function->lines = (char **) xrealloc(function->lines, max_line_count * sizeof(char *));
+          }
+        }
       }
     }
     if (read_alias)
     {
       int i;
       for (i = 0; i < alias_count; ++i)
-	process_alias(aliases[i], argc, argv, path_list, function_start_type);
+        process_alias(aliases[i], argc, argv, path_list, function_start_type);
     }
   }
 
@@ -661,16 +720,20 @@ int main(int argc, char *argv[])
     if (!*argv)
       continue;
 
-    if (read_functions && !strchr(*argv, '/'))
+    if (read_functions && !strchr(*argv, DIR_SEPARATOR))
       found_something = func_search(0, *argv, functions, function_start_type);
 
     if ((show_all || !found_something) && !path_search(0, *argv, path_list) && !found_something)
     {
-      print_fail(absolute_path_given ? strrchr(*argv, '/') + 1 : *argv, absolute_path_given ? abs_path : path_list);
+      print_fail(path_containing_separator_given ? strrchr(*argv, DIR_SEPARATOR) + 1 : *argv,
+                 path_containing_separator_given ? abs_path : path_list);
       ++fail_count;
     }
   }
-
+#ifdef _WIN32
+  if (path_list)
+    free(path_list);
+#endif
   return fail_count;
 }
 
